@@ -47,7 +47,7 @@ static void usage(void) __attribute__((noreturn));
 static void usage(void)
 {
 	fprintf(stderr, "Usage: ip -f inet6 tunnel { add | change | del | show } [ NAME ]\n");
-	fprintf(stderr, "          [ mode { ip6ip6 | ipip6 | ip6gre | any } ]\n");
+	fprintf(stderr, "          [ mode { ip6ip6 | ipip6 | ip6gre | vti6 | any } ]\n");
 	fprintf(stderr, "          [ remote ADDR local ADDR ] [ dev PHYS_DEV ]\n");
 	fprintf(stderr, "          [ encaplimit ELIM ]\n");
 	fprintf(stderr ,"          [ hoplimit TTL ] [ tclass TCLASS ] [ flowlabel FLOWLABEL ]\n");
@@ -117,7 +117,7 @@ static void print_tunnel(struct ip6_tnl_parm2 *p)
 		}
 
 		if (p->i_flags&GRE_SEQ)
-			printf("%s  Drop packets out of sequence.\n", _SL_);
+			printf("%s  Drop packets out of sequence.", _SL_);
 		if (p->i_flags&GRE_CSUM)
 			printf("%s  Checksum in received packet is required.", _SL_);
 		if (p->o_flags&GRE_SEQ)
@@ -140,7 +140,10 @@ static int parse_args(int argc, char **argv, int cmd, struct ip6_tnl_parm2 *p)
 			if (strcmp(*argv, "ipv6/ipv6") == 0 ||
 			    strcmp(*argv, "ip6ip6") == 0)
 				p->proto = IPPROTO_IPV6;
-			else if (strcmp(*argv, "ip/ipv6") == 0 ||
+			else if (strcmp(*argv, "vti6") == 0) {
+				p->proto = IPPROTO_IPV6;
+				p->i_flags |= VTI_ISVTI;
+			} else if (strcmp(*argv, "ip/ipv6") == 0 ||
 				 strcmp(*argv, "ipv4/ipv6") == 0 ||
 				 strcmp(*argv, "ipip6") == 0 ||
 				 strcmp(*argv, "ip4ip6") == 0)
@@ -152,10 +155,10 @@ static int parse_args(int argc, char **argv, int cmd, struct ip6_tnl_parm2 *p)
 				 strcmp(*argv, "any") == 0)
 				p->proto = 0;
 			else {
-                                fprintf(stderr,"Unknown tunnel mode \"%s\"\n", *argv);
-                                exit(-1);
-                        }
-                } else if (strcmp(*argv, "remote") == 0) {
+				fprintf(stderr,"Unknown tunnel mode \"%s\"\n", *argv);
+				exit(-1);
+			}
+		} else if (strcmp(*argv, "remote") == 0) {
 			inet_prefix raddr;
 			NEXT_ARG();
 			get_prefix(&raddr, *argv, preferred_family);
@@ -234,7 +237,7 @@ static int parse_args(int argc, char **argv, int cmd, struct ip6_tnl_parm2 *p)
 			if (strchr(*argv, '.'))
 				p->i_key = p->o_key = get_addr32(*argv);
 			else {
-				if (get_unsigned(&uval, *argv, 0)<0) {
+				if (get_unsigned(&uval, *argv, 0) < 0) {
 					fprintf(stderr, "invalid value of \"key\"\n");
 					exit(-1);
 				}
@@ -459,11 +462,14 @@ static int do_add(int cmd, int argc, char **argv)
 	switch (p.proto) {
 	case IPPROTO_IPIP:
 	case IPPROTO_IPV6:
+	if (p.i_flags != VTI_ISVTI)
+		return tnl_add_ioctl(cmd, "ip6_vti0", p.name, &p);
+	else
 		return tnl_add_ioctl(cmd, "ip6tnl0", p.name, &p);
 	case IPPROTO_GRE:
 		return tnl_add_ioctl(cmd, "ip6gre0", p.name, &p);
 	default:
-		fprintf(stderr, "cannot determine tunnel mode (ip6ip6, ipip6 or gre)\n");
+		fprintf(stderr, "cannot determine tunnel mode (ip6ip6, ipip6, vti6 or gre)\n");
 	}
 	return -1;
 }
@@ -480,6 +486,9 @@ static int do_del(int argc, char **argv)
 	switch (p.proto) {
 	case IPPROTO_IPIP:
 	case IPPROTO_IPV6:
+	if (p.i_flags != VTI_ISVTI)
+		return tnl_del_ioctl("ip6_vti0", p.name, &p);
+	else
 		return tnl_del_ioctl("ip6tnl0", p.name, &p);
 	case IPPROTO_GRE:
 		return tnl_del_ioctl("ip6gre0", p.name, &p);
